==> kports-0.8.1.tar.bz2: kports-0.8.1/src/bsdportglobal.h <==
/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __BSDPORTGLOBAL_H__
#define __BSDPORTGLOBAL_H__

#include "bsdport.h"
/*
#include <QString>
#include <QStringList>
#include <QFile>
#include <QFtp>
#include <QDir>
#include <QThread>
#include <QProcess>
#include <QTime>
*/
#include <QTimer>
#include <QMutex>
#include <QMutexLocker>
/* TODO MAYBE ADD squeeze to all setString-funcitons */

//class QBsdPortGlobal;
/*class QBsdPort;
class QBsdPortPkg;
class QBsdPortInstPkg;
class QBsdPortAvSrcPkg;
class QBsdPortAvBinPkg;*/

class QFtpIndexThread;
// class QInstPkgDbReader;
#include "instpkgdbreader.h"



#ifndef SPONTANEOUS
class QBsdPortGlobal : public QThread
#else
class QBsdPortGlobal : public QObject
#endif
{
Q_OBJECT
public:
//    typedef QList <QBsdPort*> QPortList;
//    typedef QList <QBsdPortPkg*> QPkgList;

    enum SEARCHFLAGS
    {
        PKGNAME = 1, CATEGORIES = 2, MAINT = 4, COMMENT = 8,
        DESCR = 16, FILES = 32
    };

    enum PKGTYPE
    {
        AVSRCPKG = 1, AVBINPKG = 2, INSTPKG = 4,
        ANYPKG = AVSRCPKG + AVBINPKG + INSTPKG
    };

private:
    /** member vars **/

    /// the list containing *ALL* ports
    QPortList m_collection;
    /// PORTNAMEs that are shared by multiple ports, e.g. "GCC"
//    QStringList m_ambigiousNames;


//    QString m_session;      /// a 6-letter random string
    QDir    m_sessionDir; /// temp()+"bpk-"+m_session

    QDir    m_portsDir,    /// portsdir, defaults to /usr/ports
//            m_portsDbDir,  /// ports db, defaults to /var/db/ports
            m_pkgDbDir;    /// pkg db, defaults to /var/db/pkg
//    QFile   m_bpkConf;     /// configuration file for var overwrites

    QString m_pkgSite;      /// PACKAGESITE as string
    QStringList m_pkgDirs;  /// places where packages might except on mirror

    QStringList m_defaultBuildEnv;

    QDate   m_updDate;      /// ignore UPDATING file warnings older than this


    QFile   m_indexFile;   /// index file, defaults to /usr/ports/INDEX

    qint64  m_bigPkgSize; /// Pkgs of this size wont be checksummed

    int     m_searchId;     /// identifier for searches in collection

    int     m_portaudit;

    /// check if mirror is valid and if need be update the INDEX from that site
    ///, NEEDS TO BE RUN ON EVERY START
/*    void    checkFtpIndex();
    QFile   m_ftpIndexFile;/// INDEX from ftp, DEFAULT: /var/db/bpk/FTPINDEX
    QFtp*   m_pkgSiteFtp;    /// PACKAGESITE-ftp
    int     m_ftpIndexState; /**  0 = not doing anything
                            1 = logging in /listing -1 = ... failed
                            2 = index exists -2 = doenst exist
                            3 = fetching -3 = ... failed
                            4 = up2date -4 = could not open file for writing
                            5 = read successfully**/
//    bool    m_finalStateReported;

    QInstPkgDbReader* m_instPkgDbReader;
    QFtpIndexThread* m_avBinThread;

    /** error pkgs (returned when illegal pkg is requested) **/
    QBsdPort*       m_errorPort;

    /** other private functions **/


//   #ifndef SPONTANEOUS
//     /// read UPDATING and add warnings if necessary (only installed versions)
//     void	readUpdFile();
//   #endif

    /// a random string of length 'length' in capital letters
    QString randomString(int length);

//    void    defaultValues();    /// set defaultValues for members
//    void    checkVars();        /// check validity of essential vars
    void    createErrPkgs();    /// duh..

    mutable QMutex  collectionLOCK;
    mutable QMutex  settingsLock;

public slots:
    void  init();

public:

    /** constructor and destructor **/
    QBsdPortGlobal();
    ~QBsdPortGlobal();

    void    readSettings();



#ifndef SPONTANEOUS
    void    run();
    void    getGoing();
#endif

    /** these functions retrieve information and are run on startup **/
    /// find ports and add AvSrcPkgs
    void	findAllAvSrc();
#ifndef SPONTANEOUS
    /// add installed pkgs to array (read output from "pkg_info -apcdDrRLso")
    void	findAllInst();
#endif

    /// get the contents of makefile-variable 'var' in dir with extra-env 'env'
    QString     getMakeEnv(const QString& path,
                           const QString& var,
                           const QStringList& env = QStringList() );
    QString     getMakeEnv(const QString& path,
                           const QStringList& vars,
                           const QStringList& env = QStringList() );
    /// run a command in foreground and return its output
    QStringList runCmd(const QString& cmd, int linesOfOutput=0);

    /// return port with corresponding origin
    QBsdPort	*portWithOrigin(const  QString& origin,
                                uint   startPos = 0,
                                int    *returnPos = 0 );


    /// return port with corresponding name (and other vars)
    const QPortList   portWithPkgName(const QString& Name,
                                      PKGTYPE    pkgtype = ANYPKG,
                                      bool withVersion = true);

    /** hardcoded options **/
    bool    const spontaneous()
    {
    #ifdef SPONTANEOUS
        return 1;
    #endif
        return 0;
    }

    /** get calls for members **/
    inline QPortList   collection()    const   { return m_collection; }
//    QStringList ambigiousNames() const  { return m_ambigiousNames; }

    inline QInstPkgDbReader* instPkgDbReader() const
    {
        return m_instPkgDbReader;
    }


    inline QDir sessionDir() const
    {
        QMutexLocker locker(&settingsLock);
        return m_sessionDir;
    }
    inline QDir portsDir() const
    {
        QMutexLocker locker(&settingsLock);
        return m_portsDir;
    }
//    QDir    portsDbDir()        const   { return m_portsDbDir; }
    inline QDir pkgDbDir() const
    {
        QMutexLocker locker(&settingsLock);
        return m_pkgDbDir;
    }
    inline QString pkgSite() const
    {
         QMutexLocker locker(&settingsLock);
        return m_pkgSite;
    }
    inline QDate updDate() const
    {
        QMutexLocker locker(&settingsLock);
        return m_updDate;
    }
    inline QStringList defaultBuildEnv() const
    {
        QMutexLocker locker(&settingsLock);
        return m_defaultBuildEnv;
    }
//  #ifndef SPONTANEOUS

    inline QFile &indexFile()
    {
        QMutexLocker locker(&settingsLock);
        return m_indexFile;
    }
    QFtpIndexThread* avBinThread() const   { return m_avBinThread; }

    inline qint64  bigPkgSize() const
    {
        QMutexLocker locker(&settingsLock);
        return m_bigPkgSize;
    }
    inline bool portaudit() const
    {
        QMutexLocker locker(&settingsLock);
        return m_portaudit;
    }

    inline QStringList pkgDirs() const
    {
        QMutexLocker locker(&settingsLock);
        return m_pkgDirs;
    }
//  #endif

    QBsdPort*       errorPort()         { return m_errorPort; }

    /** set calls for members **/

//     inline void setInstPkgDbReader(QInstPkgDbReader* newValue)
//     {
//         m_instPkgDbReader = newValue;
//         emit instPkgDbReaderChanged(newValue);
//     }


    inline void setSessionDir(const QDir& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_sessionDir = newValue;
    }
    inline void setPortsDir(const QDir& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_portsDir = newValue;
    }
    inline void setPkgDbDir(const QDir& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_pkgDbDir = newValue;
    }
    inline void setPkgSite(const QString& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_pkgSite = newValue;
    }
    inline void setUpdDate(const QDate& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_updDate = newValue;
    }
    inline void setDefaultBuildEnv(const QStringList& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_defaultBuildEnv = newValue;
    }
    inline void setIndexFile(const QFile& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_indexFile.setFileName(newValue.fileName());
    }
    inline void setBigPkgSize(qint64 newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_bigPkgSize = newValue;
    }
    inline void setPortaudit(bool newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_portaudit = newValue;
    }
    inline void setPkgDirs(const QStringList& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_pkgDirs = newValue;
    }



    /** other public functions **/
    ///good for removing stuff
    void    cleanDelete(QObject* o)
    {
        o->disconnect();
        //TODO mayeb delete o ist better
        o->deleteLater();
        o = 0;
    }

    template <typename T>
    QList<T> removeDuplicates (const QList<T>& list)
    {
        QList<T> ret;

        for (int i=0; i < list.count(); ++i)
            if (!ret.contains(list.at(i)))
                ret.append(list.at(i));
        return ret;
    }

    /// return 0 if equal, 1 if 1 is bigger, 2 if 2 is bigger
    int compareVersions(QString ver1, QString ver2)
    {
        //TODO test sections behaviour for "no results"
        if (ver1 == ver2)
            return 0;

        /// compare PORTEPOCH
        if (ver1.section(',',-1,-1) > ver2.section(',',-1,-1))
            return 1;
        else if (ver1.section(',',-1,-1) < ver2.section(',',-1,-1))
            return 2;

        /// compare places between dots from left to right
        bool stop=false;
        for (int i = 0; !stop; ++i)
        {
            if (ver1.section('.',i,i) > ver2.section('.',i,i))
                return 1;
            else if (ver1.section('.',i,i) < ver2.section('.',i,i))
                return 2;
            // verify whether section returns empty string or ""
            if ( (ver1.section('.',i,i)=="") && (ver2.section('.',i,i)=="") )
                stop=true;
        }
        emit error("compareVersions() : got no result for " + ver1 + " and" +
                   ver2 + ". Returned versions as being equal.");
        return 0;
    }

    inline void		emitWarning(const QString& s)
	{
		emit warning(s);
	}

    inline void		emitError(const QString& s)
	{
		emit error(s);
	}

    inline void		emitCritical(const QString& s)
	{
		emit critical(s);
	}

    bool lessThan(QBsdPort* p1, QBsdPort* p2);
//     {
//         return p1->origin() < p2->origin();
//     }

public slots:
    void    writeSettings();
    
    /// search the array ports fulfillining search criteria
    QPortList search(const QString term,
                     int criteria,
                     int    pkgtype = ANYPKG,
                     bool exact = false);
    QPortList search(const QString& term,
                     qint64 flags);

    /// update the ports directory using cvs/portsnap
    void    updatePorts(bool force);

  #ifdef PORTAUDIT
    /// update portaudit's database
    void    updateVulnaDb();
  #endif


    inline void    addPortToCollection(QBsdPort* p, bool sort=true)
    {
//        QMutexLocker locker(&collectionLOCK);
        m_collection.append(p);
        if (sort)
            sortCollection();
        emit newPortAdded(p);
    }


    inline void     sortCollection()
    {
        QMutexLocker locker(&collectionLOCK);
        qSort(m_collection.begin(),
              m_collection.end(),
              QBsdPort::lessThan);
        // check for ports mistakenly added twice
        QBsdPort* oldp = 0;
        foreach(QBsdPort* p, m_collection)
        {
            if (oldp)
            {
                if (oldp == p) //same object added twice
                {
                    qDebug("double-added port removed");
                    m_collection.removeOne(p);
                }
                
                if (oldp->origin() == p->origin()) // two objects for one port
                {
                    qDebug("double-referenced port removed");
                    m_collection.removeOne(p);
                    p->deleteLater();
                    //recheck stuff on oldp
                    if (!oldp->hasInst())
                        oldp->findInst();
                    if (!oldp->hasAvSrc())
                        oldp->findAvSrc();
                    if (!oldp->hasAvBin())
                        oldp->findAvBin();
                }          
            }
            oldp = p;
        }
    }

    inline void     addPkgToPort(const QString& origin,
                                 QBsdPortPkg*   pkg)
    {
        QBsdPort *p = portWithOrigin(origin);
        if (!p)
        {
            p = new QBsdPort(origin);
            addPortToCollection(p, false);
        }
        addPkgToPort(p, pkg);
    }
    
    inline void     addPkgToPort(QBsdPort*      port,
                                 QBsdPortPkg*   pkg)
    {
        if (QBsdPortInstPkg *Pkg = qobject_cast<QBsdPortInstPkg *>(pkg))
            port->setInst(Pkg);
        else if (QBsdPortAvBinPkg *Pkg = qobject_cast<QBsdPortAvBinPkg *>(pkg))
            port->setAvBin(Pkg);
        else if (QBsdPortAvSrcPkg *Pkg = qobject_cast<QBsdPortAvSrcPkg *>(pkg))
            port->setAvSrc(Pkg);
    }
    // 1 central
    inline void     emitPortState(QBsdPort *p)
    {
        if (!p)
            return;

        emit portState(p,
              p->hasInst() ? p->inst()->nv() : "",
              p->hasAvSrc() ? p->avSrc()->nv() : "",
              p->hasAvBin() ? p->avBin()->nv() : ""
              );
    }

    /// receive binary pkgs from extra thread and add to ports
    void    receiveBinaryPkgs(QList<QBsdPortAvBinPkg*> list);

signals:
/**normal signals**/
/*    void    indexFileNotFound();
    void    vulnaDbOutdated();
    void    vulnaDbUpdateSuccessful();
    void    vulnaDbUpdateUnsuccessful(QString);
    void    searchFinished(QPtrList <QBsdPort>);
    void    percentChanged(int);

    void    findAllAvBinProgress(int);
    /**	0 = deleting old data
            1 = fetching new information from server
            2 = information fetched successfully and reading it now
            3 = error during update
            4 = finished succesfully
    **/

    void    portState(QBsdPort*,        /// the port that changed
                      const QString&,   /// its installed version
                      const QString&,   /// its avsrc version
                      const QString&);  /// its avbin version

    /** concerning error handling
        syntax for the following: "calling function : description" **/
    void    warning(const QString&);   /// things that may happen
    void    error(const QString&);     /// things that really shouldnt happen
    void    critical(const QString&);  /// dangerous things that should cause the
                                /// front-end to quit immediately

    void    findAllAvSrcRunning();
    void    findAllAvSrcFinished(int);
//     void    findAllInstRunning();
//     void    findAllInstFinished(int);
//     void    instPkgDbReaderChanged(QInstPkgDbReader*);

    void    newPortAdded(QBsdPort*);

    /** concerning pkgSiteFtp **/
//    void    ftpIndexStateChanged(int);
    void    pkgSiteLoggingIn();
    void    pkgSiteFoundIndex();
    void    ftpIndexAboutToFetch();
    void    ftpIndexNowUpToDate();
    void    ftpIndexFail(const QString&);
    void    ftpIndexFetchProgress(qint64, qint64);
    void    findAllAvBinRunning();
    void    findAllAvBinFinished(int);
//    void    noAvBin();              /// for some reason no avbin can be used
    /** concerning search **/
    void    searching();
    void    searchProgress(int);
    void    searchResult(QBsdPort*);
    void    searchFinished();
  #ifdef PORTAUDIT
    /** concerning portaudit **/
    void    updatingVulnaDb();
    void    vulnaDbUpdateFinished();
    void    vulnaDbUpdateFailed(const QString&);
  #endif
};

class QFtpIndexThread : public QThread
{
Q_OBJECT
private:
    QFile   m_ftpIndexFile;/// INDEX from ftp, DEFAULT: /var/db/bpk/FTPINDEX
    QFtp*   m_pkgSiteFtp;    /// PACKAGESITE-ftp
    int     m_state; /**  0 = not doing anything
                            1 = logging in /listing -1 = ... failed
                            2 = index exists -2 = doenst exist
                            3 = fetching -3 = ... failed
                            4 = up2date -4 = could not open file for writing
                            5 = reading -5 = reading failed
                            6 = read successfully**/
    bool    m_finalStateReported;
    uint    counter;
    QString errorString;
protected:
    void    run()
    {
        exec();
    }
public:

    QFtpIndexThread()
    {
        m_ftpIndexFile.setFileName("/usr/ports/FTPINDEX");
        m_state = 0;
        m_finalStateReported = false;
        m_pkgSiteFtp = 0;
    }

    ~QFtpIndexThread()
    {
        m_pkgSiteFtp->deleteLater();
    }
    void getGoing();

    void readIndexFile();

    /** get calls for members **/
    QFtp*   pkgSiteFtp()    const   { return m_pkgSiteFtp; }
    QFile   &ftpIndexFile()         { return m_ftpIndexFile; }
    int     state()         const   { return m_state; }
    /** set calls for members **/
    void     setState(int newValue)
    {
        m_state = newValue;
        switch (newValue)
        {
            case -5: errorString = "Could not read FTPINDEX!";
                    break;
            case -4: errorString = "Could not open local FTPINDEX for writing:";
                    break;
            case -3: errorString = "Could not fetch INDEX from PACKAGESITE:";
                    break;
            case -2: errorString = "No INDEX on PACKEGESITE:";
                    break;
            case -1: errorString = "Error logging into PACKGESITE "
                                "or changing/listing Directory:";
                    break;
            case  0: break;
            case  1: emit loggingIn();
                    break;
            case  2:// qDebug("Found INDEX on PACKAGESITE");
                    emit foundIndexOnFtp();
                    break;
            case  3: //qDebug("Attempting to fetch...");
                    emit aboutToFetch();
                    break;
            case  4: //qDebug("Fetched successfully!");
                    emit localFtpIndexUpToDate();
                    pkgSiteFtp()->close();
                    pkgSiteFtp()->deleteLater();
                    readIndexFile();
                    break;
            case  5: //qDebug("Reading...");
                    emit reading();
                    break;
             case  6: //qDebug("Read successfully!");
                    m_finalStateReported = 1;
                    emit readingDone(counter);
                    break;
            default: break;
        }

        if ((newValue < 0) && !m_finalStateReported)
        {
            if (ftpIndexFile().isOpen())
                ftpIndexFile().close();
            emit error(errorString + "\n" + pkgSiteFtp()->errorString());
            emit fail(errorString + "\n" + pkgSiteFtp()->errorString());
            if (pkgSiteFtp())
            {
                pkgSiteFtp()->abort();
                pkgSiteFtp()->close();
            }
            m_finalStateReported = 1;
        }

        emit stateChanged(newValue);
    }


public slots:
    void    checkFtpIndex();
    /** for pkgsite-index **/
    void    parseUrlInfo(QUrlInfo);
    void    cmdFinished(int id,bool error);

signals:
    void    stateChanged(int);

    void    fail(const QString&);
    void    fetchProgress(qint64, qint64);
    void    loggingIn();
    void    foundIndexOnFtp();
    void    aboutToFetch();
    void    localFtpIndexUpToDate();
    void    reading();
    void    readingDone(int);

    /** concerning error handling
        syntax for the following: "calling function : description" **/
    void    warning(const QString&);   /// things that may happen
    void    error(const QString&);     /// things that really shouldnt happen
    void    critical(const QString&);  /// dangerous things that should cause the
                                /// front-end to quit immediately
};




#endif

==> kports-0.8.2.tar.bz2: kports-0.8.2/src/bsdportglobal.h <==
/***************************************************************************
 *   Copyright (C) 2005-2009 by Hannes Hauswedell                          *
 *   kports@soulrebel.in-berlin.de                                         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 3 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef __BSDPORTGLOBAL_H__
#define __BSDPORTGLOBAL_H__

#include "bsdport.h"
/*
#include <QString>
#include <QStringList>
#include <QFile>
#include <QFtp>
#include <QDir>
#include <QThread>
#include <QProcess>
#include <QTime>
*/
#include <QTimer>
#include <QMutex>
#include <QMutexLocker>
/* TODO MAYBE ADD squeeze to all setString-funcitons */

//class QBsdPortGlobal;
/*class QBsdPort;
class QBsdPortPkg;
class QBsdPortInstPkg;
class QBsdPortAvSrcPkg;
class QBsdPortAvBinPkg;*/

class QFtpIndexThread;
// class QInstPkgDbReader;
#include "instpkgdbreader.h"
#include "portscollection.h"

class QFtpIndexHandler;
class QVulnaDbHandler;
class QPortsDirHandler;

#ifndef SPONTANEOUS
class QBsdPortGlobal : public QThread
#else
class QBsdPortGlobal : public QObject
#endif
{
Q_OBJECT
public:
//    typedef QList <QBsdPort*> QPortList;
//    typedef QList <QBsdPortPkg*> QPkgList;

//     enum SEARCHFLAGS
//     {
//         PKGNAME = 1, CATEGORIES = 2, MAINT = 4, COMMENT = 8,
//         DESCR = 16, FILES = 32
//     };
// 
//     enum PKGTYPE
//     {
//         AVSRCPKG = 1, AVBINPKG = 2, INSTPKG = 4,
//         ANYPKG = AVSRCPKG + AVBINPKG + INSTPKG
//     };

private:
    /** member vars **/

    /// the list containing *ALL* ports
//     QPortList m_collection;
    QPortsCollection *m_collection;
    /// PORTNAMEs that are shared by multiple ports, e.g. "GCC"
//    QStringList m_ambigiousNames;


//    QString m_session;      /// a 6-letter random string
    QDir    m_sessionDir; /// temp()+"bpk-"+m_session

    QDir    m_portsDir,    /// portsdir, defaults to /usr/ports
//            m_portsDbDir,  /// ports db, defaults to /var/db/ports
            m_pkgDbDir;    /// pkg db, defaults to /var/db/pkg
//    QFile   m_bpkConf;     /// configuration file for var overwrites

    QString m_pkgSite;      /// PACKAGESITE as string
    QStringList m_pkgDirs;  /// places where packages might except on mirror

    QStringList m_defaultBuildEnv;

    QDate   m_updDate;      /// ignore UPDATING file warnings older than this


    QFile   m_indexFile;   /// index file, defaults to /usr/ports/INDEX

    qint64  m_bigPkgSize; /// Pkgs of this size wont be checksummed

    int     m_searchId;     /// identifier for searches in collection

    int     m_portaudit;


    QInstPkgDbReader* m_instPkgDbReader;
    QFtpIndexHandler* m_ftpIndex;
    QVulnaDbHandler*  m_vulnaDbHandler;
    QPortsDirHandler* m_portsDirHandler;
//     QFtpIndexThread* m_avBinThread;

    /** error pkgs (returned when illegal pkg is requested) **/
    QBsdPort*       m_errorPort;

    /** other private functions **/


    /// a random string of length 'length' in capital letters
    QString randomString(int length);

//    void    defaultValues();    /// set defaultValues for members
//    void    checkVars();        /// check validity of essential vars
    void    createErrPkgs();    /// duh..

    mutable QMutex  collectionLOCK;
    mutable QMutex  settingsLock;

public slots:
    void  init();

public:

    /** constructor and destructor **/
    QBsdPortGlobal();
    ~QBsdPortGlobal();

    void    readSettings();



#ifndef SPONTANEOUS
    void    run();
    void    getGoing();
#endif

    /** these functions retrieve information and are run on startup **/
    /// find ports and add AvSrcPkgs
    void	findAllAvSrc();
#ifndef SPONTANEOUS
    /// add installed pkgs to array (read output from "pkg_info -apcdDrRLso")
    void	findAllInst();
#endif

    /// get the contents of makefile-variable 'var' in dir with extra-env 'env'
    QString     getMakeEnv(const QString& path,
                           const QString& var,
                           const QStringList& env = QStringList() );
    QString     getMakeEnv(const QString& path,
                           const QStringList& vars,
                           const QStringList& env = QStringList() );
    /// run a command in foreground and return its output
    QStringList runCmd(const QString& cmd, int linesOfOutput=0);

    /** hardcoded options **/
    bool    const spontaneous()
    {
    #ifdef SPONTANEOUS
        return 1;
    #endif
        return 0;
    }

    /** get calls for members **/
    inline QPortsCollection* collection()
    {
        return m_collection;
    }
//    QStringList ambigiousNames() const  { return m_ambigiousNames; }

    inline QInstPkgDbReader* instPkgDbReader() const
    {
        return m_instPkgDbReader;
    }

    inline QFtpIndexHandler* ftpIndex() const
    {
        return m_ftpIndex;
    }

    inline QVulnaDbHandler* vulnaDbHandler() const
    {
        return m_vulnaDbHandler;
    }

    inline QPortsDirHandler* portsDirHandler() const
    {
        return m_portsDirHandler;
    }

    inline QDir sessionDir() const
    {
        QMutexLocker locker(&settingsLock);
        return m_sessionDir;
    }
    inline QDir portsDir() const
    {
        QMutexLocker locker(&settingsLock);
        return m_portsDir;
    }
//    QDir    portsDbDir()        const   { return m_portsDbDir; }
    inline QDir pkgDbDir() const
    {
        QMutexLocker locker(&settingsLock);
        return m_pkgDbDir;
    }
    inline QString pkgSite() const
    {
         QMutexLocker locker(&settingsLock);
        return m_pkgSite;
    }
    inline QDate updDate() const
    {
        QMutexLocker locker(&settingsLock);
        return m_updDate;
    }
    inline QStringList defaultBuildEnv() const
    {
        QMutexLocker locker(&settingsLock);
        return m_defaultBuildEnv;
    }
//  #ifndef SPONTANEOUS

    inline QFile &indexFile()
    {
        QMutexLocker locker(&settingsLock);
        return m_indexFile;
    }
//     QFtpIndexThread* avBinThread() const   { return m_avBinThread; }

    inline qint64  bigPkgSize() const
    {
        QMutexLocker locker(&settingsLock);
        return m_bigPkgSize;
    }
    inline bool portaudit() const
    {
        QMutexLocker locker(&settingsLock);
        return m_portaudit;
    }

    inline QStringList pkgDirs() const
    {
        QMutexLocker locker(&settingsLock);
        return m_pkgDirs;
    }
//  #endif

    QBsdPort*       errorPort()         { return m_errorPort; }

    /** set calls for members **/

    inline void setSessionDir(const QDir& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_sessionDir = newValue;
    }
    inline void setPortsDir(const QDir& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_portsDir = newValue;
    }
    inline void setPkgDbDir(const QDir& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_pkgDbDir = newValue;
    }
    inline void setPkgSite(const QString& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_pkgSite = newValue;
    }
    inline void setUpdDate(const QDate& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_updDate = newValue;
    }
    inline void setDefaultBuildEnv(const QStringList& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_defaultBuildEnv = newValue;
    }
    inline void setIndexFile(const QFile& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_indexFile.setFileName(newValue.fileName());
    }
    inline void setBigPkgSize(qint64 newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_bigPkgSize = newValue;
    }
    inline void setPortaudit(bool newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_portaudit = newValue;
    }
    inline void setPkgDirs(const QStringList& newValue)
    {
        QMutexLocker locker(&settingsLock);
        m_pkgDirs = newValue;
    }



    /** other public functions **/
    ///good for removing stuff
    void    cleanDelete(QObject* o)
    {
       //o->disconnect();
        //TODO mayeb delete o ist better
        //o->deleteLater();
       delete o;
        o = 0;
    }

    template <typename T>
    QList<T> removeDuplicates (const QList<T>& list)
    {
        QList<T> ret;

        for (int i=0; i < list.count(); ++i)
            if (!ret.contains(list.at(i)))
                ret.append(list.at(i));
        return ret;
    }

    /// return 0 if equal, 1 if 1 is bigger, 2 if 2 is bigger
    int compareVersions(QString ver1, QString ver2)
    {
        //TODO test sections behaviour for "no results"
        if (ver1 == ver2)
            return 0;

        /// compare PORTEPOCH
        if (ver1.section(',',-1,-1) > ver2.section(',',-1,-1))
            return 1;
        else if (ver1.section(',',-1,-1) < ver2.section(',',-1,-1))
            return 2;

        /// compare places between dots from left to right
        bool stop=false;
        for (int i = 0; !stop; ++i)
        {
            if (ver1.section('.',i,i) > ver2.section('.',i,i))
                return 1;
            else if (ver1.section('.',i,i) < ver2.section('.',i,i))
                return 2;
            // verify whether section returns empty string or ""
            if ( (ver1.section('.',i,i)=="") && (ver2.section('.',i,i)=="") )
                stop=true;
        }
        emit error("compareVersions() : got no result for " + ver1 + " and" +
                   ver2 + ". Returned versions as being equal.");
        return 0;
    }

    inline void		emitWarning(const QString& s)
	{
		emit warning(s);
	}

    inline void		emitError(const QString& s)
	{
		emit error(s);
	}

    inline void		emitCritical(const QString& s)
	{
		emit critical(s);
	}

    bool lessThan(QBsdPort* p1, QBsdPort* p2);
//     {
//         return p1->origin() < p2->origin();
//     }

public slots:
    void    writeSettings();

    // 1 central
    inline void     emitPortState(QBsdPort *p)
    {
        if (!p)
            return;

        emit portState(p,
              p->hasInst() ? p->inst()->nv() : "",
              p->hasAvSrc() ? p->avSrc()->nv() : "",
              p->hasAvBin() ? p->avBin()->nv() : ""
              );
    }

    void    reset();    // reread index, pkgdb & ftpindex
signals:
/**normal signals**/

    void    portState(QBsdPort*,        /// the port that changed
                      const QString&,   /// its installed version
                      const QString&,   /// its avsrc version
                      const QString&);  /// its avbin version

    /** concerning error handling
        syntax for the following: "calling function : description" **/
    void    warning(const QString&);   /// things that may happen
    void    error(const QString&);     /// things that really shouldnt happen
    void    critical(const QString&);  /// dangerous things that should cause the
                                /// front-end to quit immediately

    void    findAllAvSrcRunning();
    void    findAllAvSrcFinished(int);
};



#endif
